! $Header: /home/cvsroot/LMDZ4/libf/dyn3d/gcm.F,v 1.4 2006/04/04 15:05:16 lmdzadmin Exp $

      PROGRAM gcm

      USE IOIPSL
      use dimens_m
      use paramet_m
      use comconst
      use comdissnew

      IMPLICIT NONE

c      ......   Version  du 10/01/98    ..........

c             avec  coordonnees  verticales hybrides 
c   avec nouveaux operat. dissipation * ( gradiv2,divgrad2,nxgraro2 )

c=======================================================================
c
c   Auteur:  P. Le Van /L. Fairhead/F.Hourdin
c   -------
c
c   Objet:
c   ------
c
c   GCM LMD nouvelle grille
c
c=======================================================================
c
c  ... Dans inigeom , nouveaux calculs pour les elongations  cu , cv
c      et possibilite d'appeler une fonction f(y)  a derivee tangente
c      hyperbolique a la  place de la fonction a derivee sinusoidale.
c  ... Possibilite de choisir le schema pour l'advection de
c        q  , en modifiant iadv dans traceur.def  (MAF,10/02) .
c
c      Pour Van-Leer + Vapeur d'eau saturee, iadv(1)=4. (F.Codron,10/99)
c      Pour Van-Leer iadv=10
c
c-----------------------------------------------------------------------
c   Declarations:
c   -------------

#include "comvert.h"
#include "comgeom.h"
#include "logic.h"
#include "temps.h"
#include "control.h"
#include "description.h"
#include "com_io_dyn.h"
#include "iniprint.h"
#include "tracstoke.h"

      INTEGER         longcles
      PARAMETER     ( longcles = 20 )
      REAL  clesphy0( longcles )
      SAVE  clesphy0

      REAL zdtvr

c   variables dynamiques
      REAL vcov(ip1jm,llm),ucov(ip1jmp1,llm) ! vents covariants
      REAL teta(ip1jmp1,llm)                 ! temperature potentielle 
      REAL q(ip1jmp1,llm,nqmx)               ! champs advectes
      REAL ps(ip1jmp1)                       ! pression  au sol (Pa)

      REAL masse(ip1jmp1,llm)                ! masse d'air
      REAL phis(ip1jmp1)                     ! geopotentiel au sol

c variables dynamiques intermediaires pour le transport

c   variables pour le fichier histoire
      REAL time_0

      INTEGER i,j

      real time_step, t_wrt, t_ops

      LOGICAL call_iniphys
      data call_iniphys/.true./

      character*80 dynhist_file, dynhistave_file
      character*20 modname
      character*80 abort_message

C Calendrier
      LOGICAL true_calendar
      PARAMETER (true_calendar = .false.)

c-----------------------------------------------------------------------
c    variables pour l'initialisation de la physique :
c    ------------------------------------------------
      INTEGER ngridmx,nq
      PARAMETER( ngridmx = 2+(jjm-1)*iim - 1/jjm   )
      REAL zcufi(ngridmx),zcvfi(ngridmx)
      REAL latfi(ngridmx),lonfi(ngridmx)
      REAL airefi(ngridmx)
      SAVE latfi, lonfi, airefi

      REAL TEMP

c-----------------------------------------------------------------------
c   Initialisations:
c   ----------------

      abort_message = 'last timestep reached'
      modname = 'gcm'
      descript = 'Run GCM LMDZ'
      dynhist_file = 'dyn_hist.nc'
      dynhistave_file = 'dyn_hist_ave.nc'

c--------------------------------------------------------------------------
c   Iflag_phys controle l'appel a la physique :
c   -------------------------------------------
c      0 : pas de physique
c      1 : Normale (appel a phylmd, phymars ...)
c      2 : rappel Newtonien pour la temperature + friction au sol
      iflag_phys=1

c--------------------------------------------------------------------------
c   Lecture de l'etat initial :
c   ---------------------------
c     T : on lit start.nc
c     F : le modele s'autoinitialise avec un cas academique (iniacademic)
      read_start=.true.

c-----------------------------------------------------------------------
c   Choix du calendrier
c   -------------------

      if (true_calendar) then
        call ioconf_calendar('gregorian')
      else
        call ioconf_calendar('360d')
      endif

c----------------------------------------------------------------------
c  lecture des fichiers gcm.def ou run.def
c  ---------------------------------------
      CALL conf_gcm( 99, .TRUE. , clesphy0 )

c-----------------------------------------------------------------------
c   Initialisation des traceurs
c   ---------------------------
c  Choix du schema pour l'advection
c  dans fichier trac.def ou via INCA
       call iniadvtrac(nq)

c-----------------------------------------------------------------------
c   Lecture de l'etat initial :
c   ---------------------------

c  lecture du fichier start.nc
      if (read_start) then
         CALL dynetat0("start.nc",nqmx,vcov,ucov,
     .              teta,q,masse,ps,phis, time_0)
CC Lionel GUEZ - Begin special experiment
         print *, "This is a special experiment."
         print *, "We are setting:"
         print *, "ucov = vcov = 0, q = 0, ps = 101325"
         print *, "We are averaging 'teta' horizontally."
         ucov = 0.
         vcov = 0.
         q = 0.
c        ps = 101325.
         temp=0.
         do j=1,ip1jmp1
         temp=temp+ps(j)/float(ip1jmp1)
         enddo
         do j=1,ip1jmp1
         ps(j)=temp
         enddo
         print *,'Dans gcm ps:',ps
         do i=1,llm
         temp=0.
         do j=1,ip1jmp1
         temp=temp+teta(j,i)/float(ip1jmp1)
         enddo
         do j=1,ip1jmp1
         teta(j,i)=temp
         enddo
         enddo
         do i=1,llm
         temp=0.
         do j=1,ip1jmp1
         temp=temp+masse(j,i)/float(ip1jmp1)
         enddo
         do j=1,ip1jmp1
c        masse(j,i)=temp
         enddo
         enddo
C        Average teta over all longitudes and latitudes:
C        Average teta over all longitudes and latitudes:
c       forall(i = 1:llm) teta(:,i) = sum(teta(:,i)) / ip1jmp1
c       forall(i = 1:llm) masse(:,i) = sum(masse(:,i)) / ip1jmp1
C        (it would be better to weight each element with an associated
C         surface area)
CC End special experiment
      endif

c le cas echeant, creation d un etat initial
      IF (prt_level > 9) WRITE(lunout,*)
     .                 'AVANT iniacademic AVANT AVANT AVANT AVANT'
      if (.not.read_start) then
         CALL iniacademic(nqmx,vcov,ucov,teta,q,masse,ps,phis,time_0)
      endif

c-----------------------------------------------------------------------
c   Lecture des parametres de controle pour la simulation :
c   -------------------------------------------------------
c  on recalcule eventuellement le pas de temps

      IF(MOD(day_step,iperiod).NE.0) THEN
        abort_message = 
     .  'Il faut choisir un nb de pas par jour multiple de iperiod'
        call abort_gcm(modname,abort_message,1)
      ENDIF

      IF(MOD(day_step,iphysiq).NE.0) THEN
        abort_message = 
     * 'Il faut choisir un nb de pas par jour multiple de iphysiq'
        call abort_gcm(modname,abort_message,1)
      ENDIF

      zdtvr    = daysec/FLOAT(day_step)
        IF(dtvr.NE.zdtvr) THEN
         WRITE(lunout,*)
     .    'WARNING!!! changement de pas de temps',dtvr,'>',zdtvr
        ENDIF

C
C on remet le calendrier  zero si demande
c
      if (annee_ref .ne. anneeref .or. day_ref .ne. dayref) then
        write(lunout,*)
     .  ' Attention les dates initiales lues dans le fichier'
        write(lunout,*)
     .  ' restart ne correspondent pas a celles lues dans '
        write(lunout,*)' gcm.def'
        if (raz_date .ne. 1) then
          write(lunout,*)
     .    ' On garde les dates du fichier restart'
        else
          annee_ref = anneeref
          day_ref = dayref
          day_ini = dayref
          itau_dyn = 0
          itau_phy = 0
          time_0 = 0.
          write(lunout,*)
     .   ' On reinitialise a la date lue dans gcm.def'
        endif
      ELSE
        raz_date = 0
      endif

c-----------------------------------------------------------------------
c   Initialisation des constantes dynamiques :
c   ------------------------------------------
      dtvr = zdtvr
      CALL iniconst

c-----------------------------------------------------------------------
c   Initialisation de la geometrie :
c   --------------------------------
      CALL inigeom

c-----------------------------------------------------------------------
c   Initialisation du filtre :
c   --------------------------
      CALL inifilr
c
c-----------------------------------------------------------------------
c   Initialisation de la dissipation :
c   ----------------------------------

      CALL inidissip( lstardis, nitergdiv, nitergrot, niterh   ,
     *                tetagdiv, tetagrot , tetatemp              )

c-----------------------------------------------------------------------
c   Initialisation de la physique :
c   -------------------------------
      IF (call_iniphys.and.iflag_phys.eq.1) THEN
         latfi(1)=rlatu(1)
         lonfi(1)=0.
         zcufi(1) = cu(1)
         zcvfi(1) = cv(1)
         DO j=2,jjm
            DO i=1,iim
               latfi((j-2)*iim+1+i)= rlatu(j)
               lonfi((j-2)*iim+1+i)= rlonv(i)
               zcufi((j-2)*iim+1+i) = cu((j-1)*iip1+i)
               zcvfi((j-2)*iim+1+i) = cv((j-1)*iip1+i)
            ENDDO
         ENDDO
         latfi(ngridmx)= rlatu(jjp1)
         lonfi(ngridmx)= 0.
         zcufi(ngridmx) = cu(ip1jm+1)
         zcvfi(ngridmx) = cv(ip1jm-iim)
         CALL gr_dyn_fi(1,iip1,jjp1,ngridmx,aire,airefi)
         WRITE(lunout,*)
     .           'WARNING!!! vitesse verticale nulle dans la physique'
         CALL iniphysiq(ngridmx,llm,daysec,day_ini,dtphys ,
     ,                latfi,lonfi,airefi,zcufi,zcvfi,rad,g,r,cpp     )
         call_iniphys=.false.
      ENDIF

c  numero de stockage pour les fichiers de redemarrage:

c-----------------------------------------------------------------------
c   Initialisation des I/O :
c   ------------------------


      day_end = day_ini + nday
      WRITE(lunout,300)day_ini,day_end

      CALL dynredem0("restart.nc", day_end, phis, nqmx)

      ecripar = .TRUE.

      if ( 1.eq.1) then
      time_step = zdtvr
      t_ops = iecri * daysec
      t_wrt = iecri * daysec
      CALL inithist(dynhist_file,day_ref,annee_ref,time_step,
     .              t_ops, t_wrt, nqmx, histid, histvid)

      t_ops = iperiod * time_step
      t_wrt = periodav * daysec
      CALL initdynav(dynhistave_file,day_ref,annee_ref,time_step,
     .              t_ops, t_wrt, nqmx, histaveid)
      endif

c  Choix des frequences de stokage pour le offline
c      istdyn=day_step/4     ! stockage toutes les 6h=1jour/4
c      istdyn=day_step/12     ! stockage toutes les 2h=1jour/12
      istdyn=day_step/4     ! stockage toutes les 6h=1jour/12
      istphy=istdyn/iphysiq     


c
c-----------------------------------------------------------------------
c   Integration temporelle du modele :
c   ----------------------------------

      CALL leapfrog(ucov,vcov,teta,ps,masse,phis,nq,q,clesphy0,
     .              time_0)

 300  FORMAT('1'/,15x,'run du pas',i7,2x,'au pas',i7,2x,
     . 'c''est a dire du jour',i7,3x,'au jour',i7//)
      END
